@@ -36,7 +36,7 @@
 #include "tiledb/sm/enums/layout.h"
 #include "tiledb/sm/rtree/rtree.h"
 
-#include <catch.hpp>
+#include <test/support/tdb_catch.h>
 #include <iostream>
 
 using namespace tiledb::sm;
