@@ -41,6 +41,7 @@
 #include "tiledb/common/memory_tracker.h"
 #include "tiledb/common/status.h"
 #include "tiledb/sm/array/array_directory.h"
+#include "tiledb/sm/array/consistency.h"
 #include "tiledb/sm/array_schema/array_schema.h"
 #include "tiledb/sm/crypto/encryption_key.h"
 #include "tiledb/sm/fragment/fragment_info.h"
@@ -57,9 +58,28 @@ class FragmentMetadata;
 class StorageManager;
 enum class QueryType : uint8_t;
 
+/**
+ * Free function that returns a reference to the ConsistencyController object.
+ */
+ConsistencyController& controller();
+
 /**
  * An array object to be opened for reads/writes. An ``Array`` instance
  * is associated with the timestamp it is opened at.
+ *
+ * @invariant is_opening_or_closing_ is false outside of the body of
+ * an open or close function.
+ *
+ * @invariant is_opening_or_closing_ is true when the class is either
+ * partially open or partially closed.
+ *
+ * @invariant atomicity must be maintained between the following:
+ * 1. an open Array.
+ * 2. the is_open_ flag.
+ * 3. the existence of a ConsistencySentry object, which represents
+ * open Array registration.
+ *
+ * @invariant mtx_ must not be locked outside of the scope of a member function.
  */
 class Array {
  public:
@@ -68,7 +88,10 @@ class Array {
   /* ********************************* */
 
   /** Constructor. */
-  Array(const URI& array_uri, StorageManager* storage_manager);
+  Array(
+      const URI& array_uri,
+      StorageManager* storage_manager,
+      ConsistencyController& cc = controller());
 
   /** Destructor. */
   ~Array() = default;
@@ -181,6 +204,19 @@ class Array {
   /** Closes the array and frees all memory. */
   Status close();
 
+  /**
+   * Deletes the fragments from the Array with given URI.
+   *
+   * @param uri The uri of the Array whose fragments are to be deleted.
+   * @param timestamp_start The start timestamp at which to delete fragments.
+   * @param timestamp_end The end timestamp at which to delete fragments.
+   * @return Status
+   *
+   * @pre The Array must be open for exclusive writes
+   */
+  Status delete_fragments(
+      const URI& uri, uint64_t timestamp_start, uint64_t timstamp_end);
+
   /** Returns a constant pointer to the encryption key. */
   const EncryptionKey* encryption_key() const;
 
@@ -197,7 +233,7 @@ class Array {
   bool is_empty() const;
 
   /** Returns `true` if the array is open. */
-  bool is_open() const;
+  bool is_open();
 
   /** Returns `true` if the array is remote */
   bool is_remote() const;
@@ -205,8 +241,8 @@ class Array {
   /** Retrieves the array schema. Errors if the array is not open. */
   tuple<Status, optional<shared_ptr<ArraySchema>>> get_array_schema() const;
 
-  /** Retrieves the query type. Errors if the array is not open. */
-  Status get_query_type(QueryType* qyery_type) const;
+  /** Retrieves the query type. Throws if the array is not open. */
+  QueryType get_query_type() const;
 
   /**
    * Returns the max buffer size given a fixed-sized attribute/dimension and
@@ -409,6 +445,11 @@ class Array {
   /** Checks the config to see if refactored array open should be used. */
   bool use_refactored_array_open() const;
 
+  /** Set the query type to open the array for. */
+  inline void set_query_type(QueryType query_type) {
+    query_type_ = query_type;
+  }
+
  private:
   /* ********************************* */
   /*         PRIVATE ATTRIBUTES        */
@@ -451,8 +492,11 @@ class Array {
   /** `True` if the array has been opened. */
   std::atomic<bool> is_open_;
 
-  /** The query type the array was opened for. */
-  QueryType query_type_;
+  /** `True` if the array is currently in the process of opening or closing. */
+  std::atomic<bool> is_opening_or_closing_;
+
+  /** The query type the array was opened for. Default: READ */
+  QueryType query_type_ = QueryType::READ;
 
   /**
    * The starting timestamp between to open `open_array_` at.
@@ -512,6 +556,18 @@ class Array {
   /** Memory tracker for the array. */
   MemoryTracker memory_tracker_;
 
+  /** A reference to the object which controls the present Array instance. */
+  ConsistencyController& consistency_controller_;
+
+  /** Lifespan maintenance of an open array in the ConsistencyController. */
+  std::optional<ConsistencySentry> consistency_sentry_;
+
+  /**
+   * Mutex that protects atomicity between the existence of the
+   * ConsistencySentry registration and the is_open_ flag.
+   */
+  std::mutex mtx_;
+
   /* ********************************* */
   /*          PRIVATE METHODS          */
   /* ********************************* */
@@ -558,6 +614,19 @@ class Array {
 
   /** Computes the non-empty domain of the array. */
   Status compute_non_empty_domain();
+
+  /**
+   * Sets the array state as open.
+   *
+   * @param query_type The QueryType of the Array.
+   */
+  void set_array_open(const QueryType& query_type);
+
+  /** Sets the array state as closed.
+   *
+   * Note: the Sentry object will also be released upon Array destruction.
+   **/
+  void set_array_closed();
 };
 
 }  // namespace sm
