@@ -67,6 +67,8 @@ Status ArrayMetaConsolidator::consolidate(
     uint32_t key_length) {
   auto timer_se = stats_->start_timer("consolidate_array_meta");
 
+  check_array_uri(array_name);
+
   // Open array for reading
   auto array_uri = URI(array_name);
   Array array_for_reads(array_uri, storage_manager_);
@@ -151,17 +153,13 @@ Status ArrayMetaConsolidator::vacuum(const char* array_name) {
   // Get the array metadata URIs and vacuum file URIs to be vacuum
   auto vfs = storage_manager_->vfs();
   auto compute_tp = storage_manager_->compute_tp();
-  ArrayDirectory array_dir;
-  try {
-    array_dir = ArrayDirectory(
-        vfs,
-        compute_tp,
-        URI(array_name),
-        0,
-        std::numeric_limits<uint64_t>::max());
-  } catch (const std::logic_error& le) {
-    return LOG_STATUS(Status_ArrayDirectoryError(le.what()));
-  }
+
+  auto array_dir = ArrayDirectory(
+      vfs,
+      compute_tp,
+      URI(array_name),
+      0,
+      std::numeric_limits<uint64_t>::max());
 
   const auto& array_meta_uris_to_vacuum = array_dir.array_meta_uris_to_vacuum();
   const auto& vac_uris_to_vacuum = array_dir.array_meta_vac_uris_to_vacuum();
