@@ -43,6 +43,7 @@
 #include "tiledb/common/logger_public.h"
 #include "tiledb/sm/filesystem/azure.h"
 #include "tiledb/sm/global_state/global_state.h"
+#include "tiledb/sm/misc/parallel_functions.h"
 #include "tiledb/sm/misc/tdb_math.h"
 #include "tiledb/sm/misc/utils.h"
 
@@ -889,9 +890,11 @@ Status Azure::remove_dir(const URI& uri) const {
 
   std::vector<std::string> paths;
   RETURN_NOT_OK(ls(uri, &paths, ""));
-  for (const auto& path : paths) {
-    RETURN_NOT_OK(remove_blob(URI(path)));
-  }
+  auto status = parallel_for(thread_pool_, 0, paths.size(), [&](size_t i) {
+    RETURN_NOT_OK(remove_blob(URI(paths[i])));
+    return Status::Ok();
+  });
+  RETURN_NOT_OK(status);
 
   return Status::Ok();
 }
