@@ -230,7 +230,7 @@ class CompressionFilter : public Filter {
   Status set_option_impl(FilterOption option, const void* value) override;
 
   /** Serializes this filter's metadata to the given buffer. */
-  Status serialize_impl(Buffer* buff) const override;
+  void serialize_impl(Serializer& serializer) const override;
 
   /** Initializes the compression resource pool */
   void init_compression_resource_pool(uint64_t size) override;
