@@ -49,6 +49,7 @@
 #include "tiledb/sm/query/iquery_strategy.h"
 #include "tiledb/sm/query/query_buffer.h"
 #include "tiledb/sm/query/query_condition.h"
+#include "tiledb/sm/query/update_value.h"
 #include "tiledb/sm/query/validity_vector.h"
 #include "tiledb/sm/subarray/subarray.h"
 
@@ -135,11 +136,17 @@ class Query {
    * for the name of the new fragment to be created.
    *
    * @note Array must be a properly opened array.
+   *
+   * @param array The array that is being queried.
+   * @param fragment_uri The full URI for the new fragment. Only used for
+   * writes.
+   * @param fragment_base_uri Optional base name for new fragment. Only used for
+   *     writes and only if fragment_uri is empty.
    */
   Query(
       StorageManager* storage_manager,
       shared_ptr<Array> array,
-      URI fragment_uri = URI(""));
+      optional<std::string> fragment_name = nullopt);
 
   /** Destructor. */
   ~Query();
@@ -354,6 +361,9 @@ class Query {
   /** Returns the array schema. */
   const ArraySchema& array_schema() const;
 
+  /** Returns the array schema as a shared_ptr */
+  const std::shared_ptr<const ArraySchema> array_schema_shared() const;
+
   /** Returns the names of the buffers set by the user for the query. */
   std::vector<std::string> buffer_names() const;
 
@@ -377,6 +387,12 @@ class Query {
    */
   Status finalize();
 
+  /**
+   * Submits and finalizes a query, flushing all internal state. Applicable
+   * only to global layout writes, returns an error otherwise.
+   */
+  Status submit_and_finalize();
+
   /**
    * This is a deprecated API.
    * Retrieves the buffer of a fixed-sized attribute/dimension.
@@ -546,16 +562,19 @@ class Query {
    * Returns the condition for filtering results in a read query.
    * @return QueryCondition
    */
-  const QueryCondition* condition() const;
+  const QueryCondition& condition() const;
+
+  /**
+   * Returns the update values for an update query.
+   * @return UpdateValues
+   */
+  const std::vector<UpdateValue>& update_values() const;
 
   /** Processes a query. */
   Status process();
 
-  /** Create the strategy. */
-  Status create_strategy();
-
   /** Gets the strategy of the query. */
-  IQueryStrategy* strategy();
+  IQueryStrategy* strategy(bool skip_checks_serialization = false);
 
   /**
    * Switch the strategy depending on layout. Used by serialization.
@@ -578,6 +597,13 @@ class Query {
    */
   Status set_consolidation_with_timestamps();
 
+  /**
+   * Set the processed conditions for writes.
+   *
+   * @param processed_conditions The processed conditions.
+   */
+  void set_processed_conditions(std::vector<std::string>& processed_conditions);
+
   /**
    * Sets the config for the Query
    *
@@ -805,11 +831,6 @@ class Query {
       std::unordered_map<std::string, Subarray::ResultSize>& est_result_size,
       std::unordered_map<std::string, Subarray::MemorySize>& max_mem_size);
 
-  /**
-   * Sets the cell layout of the query without performing any checks.
-   */
-  Status set_layout_unsafe(Layout layout);
-
   /**
    * Sets the cell layout of the query.
    */
@@ -823,6 +844,19 @@ class Query {
    */
   Status set_condition(const QueryCondition& condition);
 
+  /**
+   * Adds an update value for an update query.
+   *
+   * @param field_name The attribute name.
+   * @param update_value The value to set.
+   * @param update_value_size The byte size of `update_value`.
+   * @return Status
+   */
+  Status add_update_value(
+      const char* field_name,
+      const void* update_value,
+      uint64_t update_value_size);
+
   /**
    * Set query status, needed for json deserialization
    * @param status
@@ -904,6 +938,15 @@ class Query {
   /** Returns if all ranges for this query are non overlapping. */
   tuple<Status, optional<bool>> non_overlapping_ranges();
 
+  /** Returns true if this is a dense query */
+  bool is_dense() const;
+
+  /** Returns a reference to the internal WrittenFragmentInfo list */
+  std::vector<WrittenFragmentInfo>& get_written_fragment_info();
+
+  /** Called from serialization to mark the query as remote */
+  void set_remote_query();
+
  private:
   /* ********************************* */
   /*         PRIVATE ATTRIBUTES        */
@@ -929,6 +972,9 @@ class Query {
   /** The data input to the callback function. */
   void* callback_data_;
 
+  /** The query type. */
+  QueryType type_;
+
   /** The layout of the cells in the result of the subarray. */
   Layout layout_;
 
@@ -941,9 +987,6 @@ class Query {
   /** The storage manager. */
   StorageManager* storage_manager_;
 
-  /** The query type. */
-  QueryType type_;
-
   /** The query strategy. */
   tdb_unique_ptr<IQueryStrategy> strategy_;
 
@@ -972,6 +1015,12 @@ class Query {
   /** The query condition. */
   QueryCondition condition_;
 
+  /** The update values. */
+  std::vector<UpdateValue> update_values_;
+
+  /** Set of attributes that have an update value. */
+  std::set<std::string> attributes_with_update_value_;
+
   /** The fragment metadata that this query will focus on. */
   std::vector<shared_ptr<FragmentMetadata>> fragment_metadata_;
 
@@ -1010,12 +1059,12 @@ class Query {
    */
   bool consolidation_with_timestamps_;
 
-  /** The name of the new fragment to be created for writes. */
-  URI fragment_uri_;
-
   /* Scratch space used for REST requests. */
   shared_ptr<Buffer> rest_scratch_;
 
+  /* Processed conditions, used for consolidation. */
+  std::vector<std::string> processed_conditions_;
+
   /**
    * Flag to force legacy reader when strategy gets created. This is used by
    * the serialization codebase if a query comes from an older version of the
@@ -1024,10 +1073,28 @@ class Query {
    */
   bool force_legacy_reader_;
 
+  /**
+   * The name of the new fragment to be created for writes.
+   *
+   * If not set, the fragment name will be created using the latest array
+   * timestamp and a generated UUID.
+   */
+  optional<std::string> fragment_name_;
+
+  /** It tracks if this is a remote query */
+  bool remote_query_;
+
   /* ********************************* */
   /*           PRIVATE METHODS         */
   /* ********************************* */
 
+  /**
+   * Create the strategy.
+   *
+   * @param skip_checks_serialization Skip checks during serialization.
+   */
+  Status create_strategy(bool skip_checks_serialization = false);
+
   Status check_set_fixed_buffer(const std::string& name);
 
   /** Checks if the buffers names have been appropriately set for the query. */
@@ -1090,6 +1157,26 @@ class Query {
       void** buffer_val,
       uint64_t** buffer_val_size,
       const ValidityVector** validity_vector) const;
+
+  /**
+   * Check if input buffers are tile aligned. This function should be called
+   * only for remote global order writes and it should enforce tile alignment
+   * for both dense and sparse arrays.
+   */
+  Status check_tile_alignment() const;
+
+  /**
+   * Check if input buffers are bigger than 5MB. S3 multipart upload
+   * requires each part be bigger than 5MB, except the last part.
+   * This function should be called only for remote global order writes.
+   */
+  Status check_buffer_multipart_size() const;
+
+  /**
+   * Reset coord buffer markers at end of a global write submit.
+   * This will allow for the user to properly set the next write batch.
+   */
+  void reset_coords_markers();
 };
 
 }  // namespace sm
