@@ -35,7 +35,10 @@
 #include "tiledb/sm/enums/datatype.h"
 #include "tiledb/sm/enums/query_condition_combination_op.h"
 #include "tiledb/sm/enums/query_condition_op.h"
+#include "tiledb/sm/fragment/fragment_metadata.h"
 #include "tiledb/sm/misc/utils.h"
+#include "tiledb/sm/query/readers/result_cell_slab.h"
+#include "tiledb/storage_format/uri/parse_uri.h"
 
 #include <algorithm>
 #include <functional>
@@ -54,16 +57,34 @@ namespace sm {
 QueryCondition::QueryCondition() {
 }
 
+QueryCondition::QueryCondition(const std::string& condition_marker)
+    : condition_marker_(condition_marker)
+    , condition_index_(0) {
+}
+
 QueryCondition::QueryCondition(tdb_unique_ptr<tiledb::sm::ASTNode>&& tree)
     : tree_(std::move(tree)) {
 }
 
+QueryCondition::QueryCondition(
+    const uint64_t condition_index,
+    const std::string& condition_marker,
+    tdb_unique_ptr<tiledb::sm::ASTNode>&& tree)
+    : condition_marker_(condition_marker)
+    , condition_index_(condition_index)
+    , tree_(std::move(tree)) {
+}
+
 QueryCondition::QueryCondition(const QueryCondition& rhs)
-    : tree_(rhs.tree_ == nullptr ? nullptr : rhs.tree_->clone()) {
+    : condition_marker_(rhs.condition_marker_)
+    , condition_index_(rhs.condition_index_)
+    , tree_(rhs.tree_ == nullptr ? nullptr : rhs.tree_->clone()) {
 }
 
 QueryCondition::QueryCondition(QueryCondition&& rhs)
-    : tree_(std::move(rhs.tree_)) {
+    : condition_marker_(std::move(rhs.condition_marker_))
+    , condition_index_(rhs.condition_index_)
+    , tree_(std::move(rhs.tree_)) {
 }
 
 QueryCondition::~QueryCondition() {
@@ -71,6 +92,8 @@ QueryCondition::~QueryCondition() {
 
 QueryCondition& QueryCondition::operator=(const QueryCondition& rhs) {
   if (this != &rhs) {
+    condition_marker_ = rhs.condition_marker_;
+    condition_index_ = rhs.condition_index_;
     tree_ = rhs.tree_ == nullptr ? nullptr : rhs.tree_->clone();
   }
 
@@ -78,6 +101,8 @@ QueryCondition& QueryCondition::operator=(const QueryCondition& rhs) {
 }
 
 QueryCondition& QueryCondition::operator=(QueryCondition&& rhs) {
+  condition_marker_ = std::move(rhs.condition_marker_);
+  condition_index_ = std::move(rhs.condition_index_);
   tree_ = std::move(rhs.tree_);
   return *this;
 }
@@ -135,6 +160,20 @@ std::unordered_set<std::string>& QueryCondition::field_names() const {
   return field_names_;
 }
 
+uint64_t QueryCondition::condition_timestamp() const {
+  if (condition_marker_.empty()) {
+    return 0;
+  }
+
+  std::pair<uint64_t, uint64_t> timestamps;
+  if (!utils::parse::get_timestamp_range(URI(condition_marker_), &timestamps)
+           .ok()) {
+    throw std::logic_error("Error parsing condition marker.");
+  }
+
+  return timestamps.first;
+}
+
 /** Full template specialization for `char*` and `QueryConditionOp::LT`. */
 template <>
 struct QueryCondition::BinaryCmpNullChecks<char*, QueryConditionOp::LT> {
@@ -334,6 +373,7 @@ struct QueryCondition::BinaryCmpNullChecks<T, QueryConditionOp::NE> {
 template <typename T, QueryConditionOp Op, typename CombinationOp>
 void QueryCondition::apply_ast_node(
     const tdb_unique_ptr<ASTNode>& node,
+    const std::vector<shared_ptr<FragmentMetadata>>& fragment_metadata,
     const uint64_t stride,
     const bool var_size,
     const bool nullable,
@@ -364,11 +404,26 @@ void QueryCondition::apply_ast_node(
         }
       }
     } else {
+      const auto f = result_tile->frag_idx();
+
+      // For delete timestamps conditions, if there's no delete metadata or
+      // delete condition was already processed, GT condition is always true.
+      if (field_name == constants::delete_timestamps &&
+          (!fragment_metadata[f]->has_delete_meta() ||
+           fragment_metadata[f]->get_processed_conditions_set().count(
+               condition_marker_) != 0)) {
+        assert(Op == QueryConditionOp::GT);
+        for (size_t c = starting_index; c < starting_index + length; ++c) {
+          result_cell_bitmap[c] = 1;
+        }
+        continue;
+      }
+
       const auto tile_tuple = result_tile->tile_tuple(field_name);
       uint8_t* buffer_validity = nullptr;
 
       if (nullable) {
-        const auto& tile_validity = std::get<2>(*tile_tuple);
+        const auto& tile_validity = tile_tuple->validity_tile();
         buffer_validity = static_cast<uint8_t*>(tile_validity.data());
       }
 
@@ -377,11 +432,11 @@ void QueryCondition::apply_ast_node(
       uint64_t c = 0;
 
       if (var_size) {
-        const auto& tile = std::get<1>(*tile_tuple);
+        const auto& tile = tile_tuple->var_tile();
         const char* buffer = static_cast<char*>(tile.data());
         const uint64_t buffer_size = tile.size();
 
-        const auto& tile_offsets = std::get<0>(*tile_tuple);
+        const auto& tile_offsets = tile_tuple->fixed_tile();
         const uint64_t* buffer_offsets =
             static_cast<uint64_t*>(tile_offsets.data());
         const uint64_t buffer_offsets_el =
@@ -415,32 +470,49 @@ void QueryCondition::apply_ast_node(
           ++c;
         }
       } else {
-        const auto& tile = std::get<0>(*tile_tuple);
-        const char* buffer = static_cast<char*>(tile.data());
-        const uint64_t cell_size = tile.cell_size();
-        uint64_t buffer_offset = start * cell_size;
-        const uint64_t buffer_offset_inc = stride * cell_size;
-
-        // Iterate through each cell in this slab.
-        while (c < length) {
-          const bool null_cell =
-              nullable && buffer_validity[start + c * stride] == 0;
-
-          // Get the cell value.
-          const void* const cell_value =
-              null_cell ? nullptr : buffer + buffer_offset;
-          buffer_offset += buffer_offset_inc;
-
-          // Compare the cell value against the value in the value node.
+        // For fragments without timestamps, use the fragment first timestamp
+        // as a comparison value.
+        if (field_name == constants::timestamps && tile_tuple == nullptr) {
+          auto timestamp =
+              fragment_metadata[result_tile->frag_idx()]->first_timestamp();
           const bool cmp = BinaryCmpNullChecks<T, Op>::cmp(
-              cell_value,
-              cell_size,
+              &timestamp,
+              constants::timestamp_size,
               condition_value_content,
               condition_value_size);
+          while (c < length) {
+            result_cell_bitmap[starting_index + c] = combination_op(
+                result_cell_bitmap[starting_index + c], (uint8_t)cmp);
+            c++;
+          }
+        } else {
+          const auto& tile = tile_tuple->fixed_tile();
+          const char* buffer = static_cast<char*>(tile.data());
+          const uint64_t cell_size = tile.cell_size();
+          uint64_t buffer_offset = start * cell_size;
+          const uint64_t buffer_offset_inc = stride * cell_size;
 
-          result_cell_bitmap[starting_index + c] = combination_op(
-              result_cell_bitmap[starting_index + c], (uint8_t)cmp);
-          ++c;
+          // Iterate through each cell in this slab.
+          while (c < length) {
+            const bool null_cell =
+                nullable && buffer_validity[start + c * stride] == 0;
+
+            // Get the cell value.
+            const void* const cell_value =
+                null_cell ? nullptr : buffer + buffer_offset;
+            buffer_offset += buffer_offset_inc;
+
+            // Compare the cell value against the value in the value node.
+            const bool cmp = BinaryCmpNullChecks<T, Op>::cmp(
+                cell_value,
+                cell_size,
+                condition_value_content,
+                condition_value_size);
+
+            result_cell_bitmap[starting_index + c] = combination_op(
+                result_cell_bitmap[starting_index + c], (uint8_t)cmp);
+            ++c;
+          }
         }
       }
     }
@@ -451,6 +523,7 @@ void QueryCondition::apply_ast_node(
 template <typename T, typename CombinationOp>
 void QueryCondition::apply_ast_node(
     const tdb_unique_ptr<ASTNode>& node,
+    const std::vector<shared_ptr<FragmentMetadata>>& fragment_metadata,
     const uint64_t stride,
     const bool var_size,
     const bool nullable,
@@ -462,6 +535,7 @@ void QueryCondition::apply_ast_node(
     case QueryConditionOp::LT:
       apply_ast_node<T, QueryConditionOp::LT, CombinationOp>(
           node,
+          fragment_metadata,
           stride,
           var_size,
           nullable,
@@ -473,6 +547,7 @@ void QueryCondition::apply_ast_node(
     case QueryConditionOp::LE:
       apply_ast_node<T, QueryConditionOp::LE, CombinationOp>(
           node,
+          fragment_metadata,
           stride,
           var_size,
           nullable,
@@ -484,6 +559,7 @@ void QueryCondition::apply_ast_node(
     case QueryConditionOp::GT:
       apply_ast_node<T, QueryConditionOp::GT, CombinationOp>(
           node,
+          fragment_metadata,
           stride,
           var_size,
           nullable,
@@ -495,6 +571,7 @@ void QueryCondition::apply_ast_node(
     case QueryConditionOp::GE:
       apply_ast_node<T, QueryConditionOp::GE, CombinationOp>(
           node,
+          fragment_metadata,
           stride,
           var_size,
           nullable,
@@ -506,6 +583,7 @@ void QueryCondition::apply_ast_node(
     case QueryConditionOp::EQ:
       apply_ast_node<T, QueryConditionOp::EQ, CombinationOp>(
           node,
+          fragment_metadata,
           stride,
           var_size,
           nullable,
@@ -517,6 +595,7 @@ void QueryCondition::apply_ast_node(
     case QueryConditionOp::NE:
       apply_ast_node<T, QueryConditionOp::NE, CombinationOp>(
           node,
+          fragment_metadata,
           stride,
           var_size,
           nullable,
@@ -538,6 +617,7 @@ template <typename CombinationOp>
 void QueryCondition::apply_ast_node(
     const tdb_unique_ptr<ASTNode>& node,
     const ArraySchema& array_schema,
+    const std::vector<shared_ptr<FragmentMetadata>>& fragment_metadata,
     const uint64_t stride,
     const std::vector<ResultCellSlab>& result_cell_slabs,
     CombinationOp combination_op,
@@ -564,6 +644,7 @@ void QueryCondition::apply_ast_node(
     case Datatype::INT8: {
       apply_ast_node<int8_t, CombinationOp>(
           node,
+          fragment_metadata,
           stride,
           var_size,
           nullable,
@@ -575,6 +656,7 @@ void QueryCondition::apply_ast_node(
     case Datatype::UINT8: {
       apply_ast_node<uint8_t, CombinationOp>(
           node,
+          fragment_metadata,
           stride,
           var_size,
           nullable,
@@ -586,6 +668,7 @@ void QueryCondition::apply_ast_node(
     case Datatype::INT16: {
       apply_ast_node<int16_t, CombinationOp>(
           node,
+          fragment_metadata,
           stride,
           var_size,
           nullable,
@@ -597,6 +680,7 @@ void QueryCondition::apply_ast_node(
     case Datatype::UINT16: {
       apply_ast_node<uint16_t, CombinationOp>(
           node,
+          fragment_metadata,
           stride,
           var_size,
           nullable,
@@ -608,6 +692,7 @@ void QueryCondition::apply_ast_node(
     case Datatype::INT32: {
       apply_ast_node<int32_t, CombinationOp>(
           node,
+          fragment_metadata,
           stride,
           var_size,
           nullable,
@@ -619,6 +704,7 @@ void QueryCondition::apply_ast_node(
     case Datatype::UINT32: {
       apply_ast_node<uint32_t, CombinationOp>(
           node,
+          fragment_metadata,
           stride,
           var_size,
           nullable,
@@ -630,6 +716,7 @@ void QueryCondition::apply_ast_node(
     case Datatype::INT64: {
       apply_ast_node<int64_t, CombinationOp>(
           node,
+          fragment_metadata,
           stride,
           var_size,
           nullable,
@@ -641,6 +728,7 @@ void QueryCondition::apply_ast_node(
     case Datatype::UINT64: {
       apply_ast_node<uint64_t, CombinationOp>(
           node,
+          fragment_metadata,
           stride,
           var_size,
           nullable,
@@ -652,6 +740,7 @@ void QueryCondition::apply_ast_node(
     case Datatype::FLOAT32: {
       apply_ast_node<float, CombinationOp>(
           node,
+          fragment_metadata,
           stride,
           var_size,
           nullable,
@@ -663,6 +752,7 @@ void QueryCondition::apply_ast_node(
     case Datatype::FLOAT64: {
       apply_ast_node<double, CombinationOp>(
           node,
+          fragment_metadata,
           stride,
           var_size,
           nullable,
@@ -674,6 +764,7 @@ void QueryCondition::apply_ast_node(
     case Datatype::STRING_ASCII: {
       apply_ast_node<char*, CombinationOp>(
           node,
+          fragment_metadata,
           stride,
           var_size,
           nullable,
@@ -686,6 +777,7 @@ void QueryCondition::apply_ast_node(
       if (var_size) {
         apply_ast_node<char*, CombinationOp>(
             node,
+            fragment_metadata,
             stride,
             var_size,
             nullable,
@@ -696,6 +788,7 @@ void QueryCondition::apply_ast_node(
       } else {
         apply_ast_node<char, CombinationOp>(
             node,
+            fragment_metadata,
             stride,
             var_size,
             nullable,
@@ -720,6 +813,7 @@ void QueryCondition::apply_ast_node(
     case Datatype::DATETIME_AS: {
       apply_ast_node<int64_t, CombinationOp>(
           node,
+          fragment_metadata,
           stride,
           var_size,
           nullable,
@@ -750,6 +844,7 @@ template <typename CombinationOp>
 void QueryCondition::apply_tree(
     const tdb_unique_ptr<ASTNode>& node,
     const ArraySchema& array_schema,
+    const std::vector<shared_ptr<FragmentMetadata>>& fragment_metadata,
     uint64_t stride,
     const std::vector<ResultCellSlab>& result_cell_slabs,
     CombinationOp combination_op,
@@ -758,6 +853,7 @@ void QueryCondition::apply_tree(
     apply_ast_node(
         node,
         array_schema,
+        fragment_metadata,
         stride,
         result_cell_slabs,
         combination_op,
@@ -788,6 +884,7 @@ void QueryCondition::apply_tree(
             apply_tree(
                 child,
                 array_schema,
+                fragment_metadata,
                 stride,
                 result_cell_slabs,
                 std::logical_and<uint8_t>(),
@@ -804,6 +901,7 @@ void QueryCondition::apply_tree(
             apply_tree(
                 child,
                 array_schema,
+                fragment_metadata,
                 stride,
                 result_cell_slabs,
                 std::logical_and<uint8_t>(),
@@ -827,6 +925,7 @@ void QueryCondition::apply_tree(
           apply_tree(
               child,
               array_schema,
+              fragment_metadata,
               stride,
               result_cell_slabs,
               std::logical_or<uint8_t>(),
@@ -851,6 +950,7 @@ void QueryCondition::apply_tree(
 
 Status QueryCondition::apply(
     const ArraySchema& array_schema,
+    const std::vector<shared_ptr<FragmentMetadata>>& fragment_metadata,
     std::vector<ResultCellSlab>& result_cell_slabs,
     const uint64_t stride) const {
   if (!tree_) {
@@ -866,6 +966,7 @@ Status QueryCondition::apply(
   apply_tree(
       tree_,
       array_schema,
+      fragment_metadata,
       stride,
       result_cell_slabs,
       std::logical_and<uint8_t>(),
@@ -917,17 +1018,17 @@ void QueryCondition::apply_ast_node_dense(
   const auto tile_tuple = result_tile->tile_tuple(field_name);
   uint8_t* buffer_validity = nullptr;
   if (nullable) {
-    const auto& tile_validity = std::get<2>(*tile_tuple);
+    const auto& tile_validity = tile_tuple->validity_tile();
     buffer_validity = static_cast<uint8_t*>(tile_validity.data()) + src_cell;
   }
 
   if (var_size) {
     // Get var data buffer and tile offsets buffer.
-    const auto& tile = std::get<1>(*tile_tuple);
+    const auto& tile = tile_tuple->var_tile();
     const char* buffer = static_cast<char*>(tile.data());
     const uint64_t buffer_size = tile.size();
 
-    const auto& tile_offsets = std::get<0>(*tile_tuple);
+    const auto& tile_offsets = tile_tuple->fixed_tile();
     const uint64_t* buffer_offsets =
         static_cast<uint64_t*>(tile_offsets.data()) + src_cell;
     const uint64_t buffer_offsets_el =
@@ -961,7 +1062,7 @@ void QueryCondition::apply_ast_node_dense(
     }
   } else {
     // Get the fixed size data buffers.
-    const auto& tile = std::get<0>(*tile_tuple);
+    const auto& tile = tile_tuple->fixed_tile();
     const char* buffer = static_cast<char*>(tile.data());
     const uint64_t cell_size = tile.cell_size();
     uint64_t buffer_offset = (start + src_cell) * cell_size;
@@ -1098,7 +1199,7 @@ void QueryCondition::apply_ast_node_dense(
   // Process the validity buffer now.
   if (nullable && node->get_condition_value_view().content() == nullptr) {
     const auto tile_tuple = result_tile->tile_tuple(node->get_field_name());
-    const auto& tile_validity = std::get<2>(*tile_tuple);
+    const auto& tile_validity = tile_tuple->validity_tile();
     const auto buffer_validity =
         static_cast<uint8_t*>(tile_validity.data()) + src_cell;
 
@@ -1628,17 +1729,17 @@ void QueryCondition::apply_ast_node_sparse(
   // Check if the combination op = OR and the attribute is nullable.
   if constexpr (
       std::is_same_v<CombinationOp, QCMax<BitmapType>> && nullable::value) {
-    const auto& tile_validity = std::get<2>(*tile_tuple);
+    const auto& tile_validity = tile_tuple->validity_tile();
     buffer_validity = static_cast<uint8_t*>(tile_validity.data());
   }
 
   if (var_size) {
     // Get var data buffer and tile offsets buffer.
-    const auto& tile = std::get<1>(*tile_tuple);
+    const auto& tile = tile_tuple->var_tile();
     const char* buffer = static_cast<char*>(tile.data());
     const uint64_t buffer_size = tile.size();
 
-    const auto& tile_offsets = std::get<0>(*tile_tuple);
+    const auto& tile_offsets = tile_tuple->fixed_tile();
     const uint64_t* buffer_offsets =
         static_cast<uint64_t*>(tile_offsets.data());
     const uint64_t buffer_offsets_el =
@@ -1673,7 +1774,7 @@ void QueryCondition::apply_ast_node_sparse(
     }
   } else {
     // Get the fixed size data buffers.
-    const auto& tile = std::get<0>(*tile_tuple);
+    const auto& tile = tile_tuple->fixed_tile();
     const char* buffer = static_cast<char*>(tile.data());
     const uint64_t cell_size = tile.cell_size();
     const uint64_t buffer_el = tile.size() / cell_size;
@@ -1799,7 +1900,7 @@ void QueryCondition::apply_ast_node_sparse(
   // Process the validity buffer now.
   if (nullable) {
     const auto tile_tuple = result_tile.tile_tuple(node_field_name);
-    const auto& tile_validity = std::get<2>(*tile_tuple);
+    const auto& tile_validity = tile_tuple->validity_tile();
     const auto buffer_validity = static_cast<uint8_t*>(tile_validity.data());
     const auto cell_num = result_tile.cell_num();
 
@@ -2020,18 +2121,13 @@ template <typename BitmapType>
 Status QueryCondition::apply_sparse(
     const ArraySchema& array_schema,
     ResultTile& result_tile,
-    std::vector<BitmapType>& result_bitmap,
-    uint64_t* cell_count) {
+    std::vector<BitmapType>& result_bitmap) {
   apply_tree_sparse<BitmapType>(
       tree_,
       array_schema,
       result_tile,
       std::multiplies<BitmapType>(),
       result_bitmap);
-  if (cell_count != nullptr) {
-    *cell_count =
-        std::accumulate(result_bitmap.begin(), result_bitmap.end(), 0);
-  }
 
   return Status::Ok();
 }
@@ -2044,20 +2140,22 @@ const tdb_unique_ptr<ASTNode>& QueryCondition::ast() const {
   return tree_;
 }
 
+const std::string& QueryCondition::condition_marker() const {
+  return condition_marker_;
+}
+
+uint64_t QueryCondition::condition_index() const {
+  return condition_index_;
+}
+
 void QueryCondition::set_ast(tdb_unique_ptr<ASTNode>&& ast) {
   tree_ = std::move(ast);
 }
 
 // Explicit template instantiations.
 template Status QueryCondition::apply_sparse<uint8_t>(
-    const ArraySchema& array_schema,
-    ResultTile&,
-    std::vector<uint8_t>&,
-    uint64_t*);
+    const ArraySchema& array_schema, ResultTile&, std::vector<uint8_t>&);
 template Status QueryCondition::apply_sparse<uint64_t>(
-    const ArraySchema& array_schema,
-    ResultTile&,
-    std::vector<uint64_t>&,
-    uint64_t*);
+    const ArraySchema& array_schema, ResultTile&, std::vector<uint64_t>&);
 }  // namespace sm
 }  // namespace tiledb
