@@ -90,6 +90,7 @@ void ResultTile::swap(ResultTile& tile) {
   std::swap(tile_idx_, tile.tile_idx_);
   std::swap(attr_tiles_, tile.attr_tiles_);
   std::swap(timestamps_tile_, tile.timestamps_tile_);
+  std::swap(delete_timestamps_tile_, tile.delete_timestamps_tile_);
   std::swap(coords_tile_, tile.coords_tile_);
   std::swap(coord_tiles_, tile.coord_tiles_);
   std::swap(compute_results_dense_func_, tile.compute_results_dense_func_);
@@ -112,16 +113,18 @@ bool ResultTile::operator==(const ResultTile& rt) const {
 }
 
 uint64_t ResultTile::cell_num() const {
-  if (!std::get<0>(coord_tiles_[0].second).empty())
-    return std::get<0>(coord_tiles_[0].second).cell_num();
+  if (coord_tiles_[0].second.has_value()) {
+    return coord_tiles_[0].second->fixed_tile().cell_num();
+  }
 
-  if (!std::get<0>(coords_tile_).empty())
-    return std::get<0>(coords_tile_).cell_num();
+  if (coords_tile_.has_value()) {
+    return coords_tile_->fixed_tile().cell_num();
+  }
 
   if (!attr_tiles_.empty()) {
     for (const auto& attr : attr_tiles_) {
       if (attr.second.has_value()) {
-        return std::get<0>(attr.second.value()).cell_num();
+        return attr.second.value().fixed_tile().cell_num();
       }
     }
   }
@@ -132,24 +135,24 @@ uint64_t ResultTile::cell_num() const {
 void ResultTile::erase_tile(const std::string& name) {
   // Handle zipped coordinates tiles
   if (name == constants::coords) {
-    coords_tile_ = TileTuple(Tile(), Tile(), Tile());
+    coords_tile_.reset();
     return;
   }
 
   if (name == constants::timestamps) {
-    timestamps_tile_ = TileTuple(Tile(), Tile(), Tile());
+    timestamps_tile_.reset();
     return;
   }
 
   if (name == constants::delete_timestamps) {
-    delete_timestamps_tile_ = TileTuple(Tile(), Tile(), Tile());
+    delete_timestamps_tile_.reset();
     return;
   }
 
   // Handle dimension tile
   for (auto& ct : coord_tiles_) {
-    if (ct.first == name) {
-      ct.second = TileTuple(Tile(), Tile(), Tile());
+    if (ct.second.has_value() && ct.first == name) {
+      ct.second.reset();
       return;
     }
   }
@@ -157,40 +160,48 @@ void ResultTile::erase_tile(const std::string& name) {
   // Handle attribute tile
   for (auto& at : attr_tiles_) {
     if (at.first == name) {
-      at.second = TileTuple(Tile(), Tile(), Tile());
+      at.second.reset();
       return;
     }
   }
 }
 
-void ResultTile::init_attr_tile(const std::string& name) {
+void ResultTile::init_attr_tile(
+    const std::string& name, bool var_size, bool nullable) {
   // Nothing to do for the special zipped coordinates tile
   if (name == constants::coords) {
+    coords_tile_ = TileTuple(false, false);
     return;
   }
 
   if (name == constants::timestamps) {
-    timestamps_tile_ = TileTuple(Tile(), Tile(), Tile());
+    timestamps_tile_ = TileTuple(false, false);
     return;
   }
 
   if (name == constants::delete_timestamps) {
-    delete_timestamps_tile_ = TileTuple(Tile(), Tile(), Tile());
+    delete_timestamps_tile_ = TileTuple(false, false);
+    return;
+  }
+
+  if (name == constants::delete_condition_index) {
+    delete_condition_index_tile_ = TileTuple(false, false);
     return;
   }
 
   // Handle attributes
   for (auto& at : attr_tiles_) {
     if (at.first == name && at.second == nullopt) {
-      at.second = TileTuple(Tile(), Tile(), Tile());
+      at.second = TileTuple(var_size, nullable);
       return;
     }
   }
 }
 
-void ResultTile::init_coord_tile(const std::string& name, unsigned dim_idx) {
-  coord_tiles_[dim_idx] = std::pair<std::string, TileTuple>(
-      name, TileTuple(Tile(), Tile(), Tile()));
+void ResultTile::init_coord_tile(
+    const std::string& name, bool var_size, unsigned dim_idx) {
+  coord_tiles_[dim_idx] =
+      std::pair<std::string, TileTuple>(name, TileTuple(var_size, false));
 
   // When at least one unzipped coordinate has been initialized, we will
   // use the unzipped `coord()` implementation.
@@ -199,56 +210,63 @@ void ResultTile::init_coord_tile(const std::string& name, unsigned dim_idx) {
 
 ResultTile::TileTuple* ResultTile::tile_tuple(const std::string& name) {
   // Handle zipped coordinates tile
-  if (name == constants::coords) {
-    return &coords_tile_;
+  if (coords_tile_.has_value() && name == constants::coords) {
+    return &coords_tile_.value();
   }
 
   // Handle timestamps tile
-  if (name == constants::timestamps) {
-    return &*timestamps_tile_;
+  if (timestamps_tile_.has_value() && name == constants::timestamps) {
+    return &timestamps_tile_.value();
   }
 
   // Handle delete timestamps tile
-  if (name == constants::delete_timestamps) {
-    return &*delete_timestamps_tile_;
+  if (delete_timestamps_tile_.has_value() &&
+      name == constants::delete_timestamps) {
+    return &delete_timestamps_tile_.value();
+  }
+
+  if (delete_condition_index_tile_.has_value() &&
+      name == constants::delete_condition_index) {
+    return &delete_condition_index_tile_.value();
   }
 
   // Handle attribute tile
   for (auto& at : attr_tiles_) {
-    if (at.first == name && at.second.has_value())
+    if (at.first == name && at.second.has_value()) {
       return &(at.second.value());
+    }
   }
 
   // Handle separate coordinates tile
   for (auto& ct : coord_tiles_) {
-    if (ct.first == name)
-      return &(ct.second);
+    if (ct.second.has_value() && ct.first == name)
+      return &(ct.second.value());
   }
 
   return nullptr;
 }
 
 const void* ResultTile::unzipped_coord(uint64_t pos, unsigned dim_idx) const {
-  const auto& coord_tile = std::get<0>(coord_tiles_[dim_idx].second);
+  const auto& coord_tile = coord_tiles_[dim_idx].second->fixed_tile();
   const uint64_t offset = pos * coord_tile.cell_size();
   void* const ret = static_cast<char*>(coord_tile.data()) + offset;
   return ret;
 }
 
 const void* ResultTile::zipped_coord(uint64_t pos, unsigned dim_idx) const {
-  auto coords_size = std::get<0>(coords_tile_).cell_size();
+  auto coords_size = coords_tile_->fixed_tile().cell_size();
   auto coord_size =
-      coords_size / std::get<0>(coords_tile_).zipped_coords_dim_num();
+      coords_size / coords_tile_->fixed_tile().zipped_coords_dim_num();
   const uint64_t offset = pos * coords_size + dim_idx * coord_size;
   void* const ret =
-      static_cast<char*>(std::get<0>(coords_tile_).data()) + offset;
+      static_cast<char*>(coords_tile_->fixed_tile().data()) + offset;
   return ret;
 }
 
 std::string_view ResultTile::coord_string(
     uint64_t pos, unsigned dim_idx) const {
-  const auto& coord_tile_off = std::get<0>(coord_tiles_[dim_idx].second);
-  const auto& coord_tile_val = std::get<1>(coord_tiles_[dim_idx].second);
+  const auto& coord_tile_off = coord_tiles_[dim_idx].second->fixed_tile();
+  const auto& coord_tile_val = coord_tiles_[dim_idx].second->var_tile();
   auto cell_num = coord_tile_off.cell_num();
   auto val_size = coord_tile_val.size();
 
@@ -274,13 +292,14 @@ std::string_view ResultTile::coord_string(
 
 uint64_t ResultTile::coord_size(unsigned dim_idx) const {
   // Handle zipped coordinate tiles
-  if (!std::get<0>(coords_tile_).empty())
-    return std::get<0>(coords_tile_).cell_size() /
-           std::get<0>(coords_tile_).zipped_coords_dim_num();
+  if (coords_tile_.has_value()) {
+    return coords_tile_->fixed_tile().cell_size() /
+           coords_tile_->fixed_tile().zipped_coords_dim_num();
+  }
 
   // Handle separate coordinate tiles
   assert(dim_idx < coord_tiles_.size());
-  return std::get<0>(coord_tiles_[dim_idx].second).cell_size();
+  return coord_tiles_[dim_idx].second->fixed_tile().cell_size();
 }
 
 bool ResultTile::same_coords(
@@ -304,7 +323,7 @@ bool ResultTile::same_coords(uint64_t pos_a, uint64_t pos_b) const {
 }
 
 uint64_t ResultTile::timestamp(uint64_t pos) {
-  const auto& tile = std::get<0>(*this->tile_tuple(constants::timestamps));
+  const auto& tile = this->tile_tuple(constants::timestamps)->fixed_tile();
   return tile.data_as<uint64_t>()[pos];
 }
 
@@ -338,24 +357,24 @@ Status ResultTile::read(
   // or coordinates have been fetched as zipped
   if ((!is_dim && name != constants::coords && !use_fragment_ts) ||
       (is_dim && !coord_tiles_[0].first.empty()) ||
-      (name == constants::coords && !std::get<0>(coords_tile_).empty())) {
-    const auto& tile = std::get<0>(*this->tile_tuple(name));
+      (name == constants::coords && coords_tile_.has_value())) {
+    const auto& tile = this->tile_tuple(name)->fixed_tile();
     auto cell_size = tile.cell_size();
     auto nbytes = len * cell_size;
     auto offset = pos * cell_size;
     return tile.read(buffer, offset, nbytes);
   } else if (
       name == constants::coords && !coord_tiles_[0].first.empty() &&
-      std::get<0>(coords_tile_).empty()) {
+      !coords_tile_.has_value()) {
     // Special case where zipped coordinates are requested, but the
     // result tile stores separate coordinates
     auto dim_num = coord_tiles_.size();
-    assert(std::get<0>(coords_tile_).empty());
+    assert(!coords_tile_.has_value());
     auto buff = static_cast<unsigned char*>(buffer);
     auto buff_offset = 0;
     for (uint64_t c = 0; c < len; ++c) {
       for (unsigned d = 0; d < dim_num; ++d) {
-        auto& coord_tile = std::get<0>(coord_tiles_[d].second);
+        auto& coord_tile = coord_tiles_[d].second->fixed_tile();
         auto cell_size = coord_tile.cell_size();
         auto tile_offset = (pos + c) * cell_size;
         RETURN_NOT_OK(
@@ -373,7 +392,7 @@ Status ResultTile::read(
   } else {
     // Last case which is zipped coordinates but split buffers
     // This is only for backwards compatibility of pre format 5 (v2.0) arrays
-    assert(!std::get<0>(coords_tile_).empty());
+    assert(coords_tile_.has_value());
     assert(name != constants::coords);
     int dim_offset = 0;
     for (uint32_t i = 0; i < domain_->dim_num(); ++i) {
@@ -383,12 +402,12 @@ Status ResultTile::read(
       }
     }
     auto buff = static_cast<unsigned char*>(buffer);
-    auto cell_size = std::get<0>(coords_tile_).cell_size();
+    auto cell_size = coords_tile_->fixed_tile().cell_size();
     auto dim_size = cell_size / domain_->dim_num();
     uint64_t offset = pos * cell_size + dim_size * dim_offset;
     for (uint64_t c = 0; c < len; ++c) {
-      RETURN_NOT_OK(std::get<0>(coords_tile_)
-                        .read(buff + (c * dim_size), offset, dim_size));
+      RETURN_NOT_OK(coords_tile_->fixed_tile().read(
+          buff + (c * dim_size), offset, dim_size));
       offset += cell_size;
     }
   };
@@ -403,8 +422,8 @@ Status ResultTile::read_nullable(
     uint64_t pos,
     uint64_t len,
     void* buffer_validity) {
-  const auto& tile = std::get<0>(*this->tile_tuple(name));
-  const auto& tile_validity = std::get<2>(*this->tile_tuple(name));
+  const auto& tile = this->tile_tuple(name)->fixed_tile();
+  const auto& tile_validity = this->tile_tuple(name)->validity_tile();
 
   auto cell_size = tile.cell_size();
   auto validity_cell_size = tile_validity.cell_size();
@@ -426,18 +445,18 @@ Status ResultTile::read_nullable(
 }
 
 bool ResultTile::stores_zipped_coords() const {
-  return !std::get<0>(coords_tile_).empty();
+  return coords_tile_.has_value();
 }
 
 const Tile& ResultTile::zipped_coords_tile() const {
   assert(stores_zipped_coords());
-  return std::get<0>(coords_tile_);
+  return coords_tile_->fixed_tile();
 }
 
 const ResultTile::TileTuple& ResultTile::coord_tile(unsigned dim_idx) const {
   assert(!stores_zipped_coords());
   assert(!coord_tiles_.empty());
-  return coord_tiles_[dim_idx].second;
+  return coord_tiles_[dim_idx].second.value();
 }
 
 template <class T>
@@ -459,7 +478,7 @@ void ResultTile::compute_results_dense(
 
   // Handle separate coordinate tiles
   if (!stores_zipped_coords) {
-    const auto& coord_tile = std::get<0>(result_tile->coord_tile(dim_idx));
+    const auto& coord_tile = result_tile->coord_tile(dim_idx).fixed_tile();
 
     auto coords = (const T*)coord_tile.data();
     T c;
@@ -479,7 +498,7 @@ void ResultTile::compute_results_dense(
               overwritten = true;
               for (unsigned d = 0; d < dim_num; ++d) {
                 const auto& coord_tile =
-                    std::get<0>(result_tile->coord_tile(dim_idx));
+                    result_tile->coord_tile(dim_idx).fixed_tile();
                 const T* const coords =
                     static_cast<const T*>(coord_tile.data());
                 auto c_d = coords[pos];
@@ -576,11 +595,11 @@ void ResultTile::compute_results_sparse<char>(
   const auto& coord_tile = result_tile->coord_tile(dim_idx);
 
   // Get offset buffer
-  const auto& coord_tile_off = std::get<0>(coord_tile);
+  const auto& coord_tile_off = coord_tile.fixed_tile();
   auto buff_off = static_cast<const uint64_t*>(coord_tile_off.data());
 
   // Get string buffer
-  const auto& coord_tile_str = std::get<1>(coord_tile);
+  const auto& coord_tile_str = coord_tile.var_tile();
   auto buff_str = static_cast<const char*>(coord_tile_str.data());
   auto buff_str_size = coord_tile_str.size();
 
@@ -739,7 +758,7 @@ void ResultTile::compute_results_sparse(
 
   // Handle separate coordinate tiles
   if (!stores_zipped_coords) {
-    const auto& coord_tile = std::get<0>(result_tile->coord_tile(dim_idx));
+    const auto& coord_tile = result_tile->coord_tile(dim_idx).fixed_tile();
     const T* const coords = static_cast<const T*>(coord_tile.data());
     for (uint64_t pos = 0; pos < coords_num; ++pos) {
       c = coords[pos];
@@ -781,7 +800,7 @@ void ResultTile::compute_results_count_sparse_string_range(
     uint64_t c_size = (pos < cell_num - 1) ? buff_off[pos + 1] - c_offset :
                                              buff_str_size - c_offset;
 
-    std::string_view str(buff_str + c_offset, c_size);
+    const std::string_view str(buff_str + c_offset, c_size);
 
     // Binary search to find the first range containing the cell.
     auto it = std::lower_bound(
@@ -821,9 +840,8 @@ void ResultTile::compute_results_count_sparse_string_range(
     // dim.
     uint64_t count = 0;
     for (uint64_t j = start_range_idx; j < end_range_idx; ++j) {
-      auto& range = cached_ranges[j];
-      count += str_coord_intersects(
-          c_offset, c_size, buff_str, range.first, range.second);
+      const auto& range = cached_ranges[j];
+      count += str >= range.first && str <= range.second;
     }
 
     // Multiply the past count by this dimension's count.
@@ -854,11 +872,11 @@ void ResultTile::compute_results_count_sparse_string(
   const auto& coord_tile = result_tile->coord_tile(dim_idx);
 
   // Get offset buffer
-  const auto& coord_tile_off = std::get<0>(coord_tile);
+  const auto& coord_tile_off = coord_tile.fixed_tile();
   auto buff_off = static_cast<const uint64_t*>(coord_tile_off.data());
 
   // Get string buffer
-  const auto& coord_tile_str = std::get<1>(coord_tile);
+  const auto& coord_tile_str = coord_tile.var_tile();
   auto buff_str = static_cast<const char*>(coord_tile_str.data());
   auto buff_str_size = coord_tile_str.size();
 
@@ -1034,7 +1052,7 @@ void ResultTile::compute_results_count_sparse(
 
   // Handle separate coordinate tiles.
   if (!stores_zipped_coords) {
-    const auto& coord_tile = std::get<0>(result_tile->coord_tile(dim_idx));
+    const auto& coord_tile = result_tile->coord_tile(dim_idx).fixed_tile();
     const T* const coords = static_cast<const T*>(coord_tile.data());
     {
       // Iterate over all cells.
