@@ -74,7 +74,8 @@ class SparseUnorderedWithDupsReader : public SparseIndexReaderBase,
       std::unordered_map<std::string, QueryBuffer>& buffers,
       Subarray& subarray,
       Layout layout,
-      QueryCondition& condition);
+      QueryCondition& condition,
+      bool skip_checks_serialization = false);
 
   /** Destructor. */
   ~SparseUnorderedWithDupsReader() = default;
@@ -135,13 +136,6 @@ class SparseUnorderedWithDupsReader : public SparseIndexReaderBase,
    */
   QueryStatusDetailsReason status_incomplete_reason() const;
 
-  /**
-   * Initializes the reader.
-   *
-   * @return Status.
-   */
-  Status init();
-
   /**
    * Initialize the memory budget variables.
    *
@@ -195,7 +189,7 @@ class SparseUnorderedWithDupsReader : public SparseIndexReaderBase,
    * @param f Fragment index.
    * @param t Tile index.
    * @param last_t Last tile index.
-   * @param array_schema Array schema.
+   * @param frag_md Fragment metadata.
    *
    * @return buffers_full, new_var_buffer_size, new_result_tiles_size.
    */
@@ -206,7 +200,7 @@ class SparseUnorderedWithDupsReader : public SparseIndexReaderBase,
       const unsigned f,
       const uint64_t t,
       const uint64_t last_t,
-      const ArraySchema& array_schema);
+      const FragmentMetadata& frag_md);
 
   /**
    * Create the result tiles.
