@@ -298,9 +298,11 @@ Status DenseTiler<T>::get_tile(
     memset(tile.validity_tile().data(), 0, tile_size);
 
     // Copy tile from buffer
-    return copy_tile(id, cell_size, buff, tile.validity_tile());
+    RETURN_NOT_OK(copy_tile(id, cell_size, buff, tile.validity_tile()));
   }
 
+  compute_tile_metadata(name, id, tile);
+
   return Status::Ok();
 }
 
@@ -481,8 +483,9 @@ std::vector<std::array<T, 2>> DenseTiler<T>::tile_subarray(uint64_t id) const {
 
   // Get the tile coordinates in the array tile domain
   std::vector<uint64_t> tile_coords_in_dom(dim_num);
-  for (unsigned d = 0; d < dim_num; ++d)
+  for (unsigned d = 0; d < dim_num; ++d) {
     tile_coords_in_dom[d] = tile_coords_in_sub[d] + first_sub_tile_coords_[d];
+  }
 
   // Calculate tile subarray based on the tile coordinates in the domain
   std::vector<std::array<T, 2>> ret(dim_num);
@@ -509,11 +512,13 @@ Status DenseTiler<T>::copy_tile(
   auto tile_offset = copy_plan.tile_start_el_ * cell_size;
   auto copy_nbytes = copy_plan.copy_el_ * cell_size;
   auto sub_strides_nbytes = copy_plan.sub_strides_el_;
-  for (auto& bsn : sub_strides_nbytes)
+  for (auto& bsn : sub_strides_nbytes) {
     bsn *= cell_size;
+  }
   auto tile_strides_nbytes = copy_plan.tile_strides_el_;
-  for (auto& tsn : tile_strides_nbytes)
+  for (auto& tsn : tile_strides_nbytes) {
     tsn *= cell_size;
+  }
   const auto& dim_ranges = copy_plan.dim_ranges_;
   auto first_d = copy_plan.first_d_;
   auto dim_num = (int64_t)dim_ranges.size();
@@ -521,14 +526,17 @@ Status DenseTiler<T>::copy_tile(
 
   // Auxiliary information needed in the copy loop
   std::vector<uint64_t> tile_offsets(dim_num);
-  for (int64_t i = 0; i < dim_num; ++i)
+  for (int64_t i = 0; i < dim_num; ++i) {
     tile_offsets[i] = tile_offset;
+  }
   std::vector<uint64_t> sub_offsets(dim_num);
-  for (int64_t i = 0; i < dim_num; ++i)
+  for (int64_t i = 0; i < dim_num; ++i) {
     sub_offsets[i] = sub_offset;
+  }
   std::vector<uint64_t> cell_coords(dim_num);
-  for (int64_t i = 0; i < dim_num; ++i)
+  for (int64_t i = 0; i < dim_num; ++i) {
     cell_coords[i] = dim_ranges[i][0];
+  }
 
   // Perform the tile copy (always in row-major order)
   auto d = dim_num - 1;
@@ -541,15 +549,17 @@ Status DenseTiler<T>::copy_tile(
     auto last_dim_changed = d;
     for (; last_dim_changed >= 0; --last_dim_changed) {
       ++cell_coords[last_dim_changed];
-      if (cell_coords[last_dim_changed] > dim_ranges[last_dim_changed][1])
+      if (cell_coords[last_dim_changed] > dim_ranges[last_dim_changed][1]) {
         cell_coords[last_dim_changed] = dim_ranges[last_dim_changed][0];
-      else
+      } else {
         break;
+      }
     }
 
     // Check if copy loop is done
-    if (last_dim_changed < 0)
+    if (last_dim_changed < 0) {
       break;
+    }
 
     // Update the offsets
     tile_offsets[last_dim_changed] +=
@@ -565,6 +575,74 @@ Status DenseTiler<T>::copy_tile(
   return Status::Ok();
 }
 
+template <class T>
+Status DenseTiler<T>::compute_tile_metadata(
+    const std::string& name, uint64_t id, WriterTile& tile) const {
+  // Calculate copy plan
+  const CopyPlan copy_plan = this->copy_plan(id);
+
+  const auto type = array_schema_.type(name);
+  const auto is_dim = array_schema_.is_dim(name);
+  const bool var = array_schema_.var_size(name);
+  const auto cell_size = array_schema_.cell_size(name);
+  const auto cell_val_num = array_schema_.cell_val_num(name);
+  TileMetadataGenerator md_generator(
+      type, is_dim, var, cell_size, cell_val_num);
+
+  // For easy reference
+  auto tile_offset = copy_plan.tile_start_el_;
+  auto sub_strides_nbytes = copy_plan.sub_strides_el_;
+  auto tile_strides_nbytes = copy_plan.tile_strides_el_;
+  const auto& dim_ranges = copy_plan.dim_ranges_;
+  auto first_d = copy_plan.first_d_;
+  auto dim_num = (int64_t)dim_ranges.size();
+  assert(dim_num > 0);
+
+  // Auxiliary information needed in the copy loop
+  std::vector<uint64_t> tile_offsets(dim_num);
+  for (int64_t i = 0; i < dim_num; ++i) {
+    tile_offsets[i] = tile_offset;
+  }
+  std::vector<uint64_t> cell_coords(dim_num);
+  for (int64_t i = 0; i < dim_num; ++i) {
+    cell_coords[i] = dim_ranges[i][0];
+  }
+
+  // Perform the tile copy (always in row-major order)
+  auto d = dim_num - 1;
+  while (true) {
+    // Copy a slab
+    md_generator.process_cell_slab(
+        tile, tile_offsets[d], tile_offsets[d] + copy_plan.copy_el_);
+
+    // Advance cell coordinates, tile and buffer offsets
+    auto last_dim_changed = d;
+    for (; last_dim_changed >= 0; --last_dim_changed) {
+      ++cell_coords[last_dim_changed];
+      if (cell_coords[last_dim_changed] > dim_ranges[last_dim_changed][1]) {
+        cell_coords[last_dim_changed] = dim_ranges[last_dim_changed][0];
+      } else {
+        break;
+      }
+    }
+
+    // Check if copy loop is done
+    if (last_dim_changed < 0) {
+      break;
+    }
+
+    // Update the offsets
+    tile_offsets[last_dim_changed] +=
+        tile_strides_nbytes[last_dim_changed + first_d];
+    for (auto i = last_dim_changed + 1; i < dim_num; ++i) {
+      tile_offsets[i] = tile_offsets[i - 1];
+    }
+  }
+
+  md_generator.set_tile_metadata(tile);
+  return Status::Ok();
+}
+
 // Explicit template instantiations
 template class DenseTiler<int8_t>;
 template class DenseTiler<uint8_t>;
