@@ -117,8 +117,6 @@ Status array_to_capnp(
     Array* array,
     capnp::Array::Builder* array_builder,
     const bool client_side) {
-  // Currently unused
-  (void)client_side;
   // The serialized URI is set if it exists
   // this is used for backwards compatibility with pre TileDB 2.5 clients that
   // want to serialized a query object TileDB >= 2.5 no longer needs to send the
@@ -200,23 +198,23 @@ Status array_from_capnp(
     if (all_schemas_reader.hasEntries()) {
       auto entries = array_reader.getArraySchemasAll().getEntries();
       for (auto array_schema_build : entries) {
-        tdb_unique_ptr<ArraySchema> schema;
-        RETURN_NOT_OK(
-            array_schema_from_capnp(array_schema_build.getValue(), &schema));
-        schema->set_array_uri(array->array_uri());
-        all_schemas[array_schema_build.getKey()] = std::move(schema);
+        auto schema{array_schema_from_capnp(
+            array_schema_build.getValue(), array->array_uri())};
+        schema.set_array_uri(array->array_uri());
+        all_schemas[array_schema_build.getKey()] =
+            make_shared<ArraySchema>(HERE(), schema);
       }
     }
     array->set_array_schemas_all(all_schemas);
   }
 
   if (array_reader.hasArraySchemaLatest()) {
-    tdb_unique_ptr<ArraySchema> array_schema_latest;
     auto array_schema_latest_reader = array_reader.getArraySchemaLatest();
-    RETURN_NOT_OK(array_schema_from_capnp(
-        array_schema_latest_reader, &array_schema_latest));
-    array_schema_latest->set_array_uri(array->array_uri());
-    array->set_array_schema_latest(std::move(array_schema_latest));
+    auto array_schema_latest{array_schema_from_capnp(
+        array_schema_latest_reader, array->array_uri())};
+    array_schema_latest.set_array_uri(array->array_uri());
+    array->set_array_schema_latest(
+        make_shared<ArraySchema>(HERE(), array_schema_latest));
   }
 
   if (array_reader.hasNonEmptyDomain()) {
@@ -245,6 +243,8 @@ Status array_open_to_capnp(
   auto config = array.config();
   RETURN_NOT_OK(config_to_capnp(&config, &config_builder));
 
+  array_open_builder->setQueryType(query_type_str(array.get_query_type()));
+
   return Status::Ok();
 }
 
@@ -262,6 +262,13 @@ Status array_open_from_capnp(
     RETURN_NOT_OK(array->set_config(*decoded_config));
   }
 
+  if (array_open_reader.hasQueryType()) {
+    auto query_type_str = array_open_reader.getQueryType();
+    QueryType query_type = QueryType::READ;
+    RETURN_NOT_OK(query_type_enum(query_type_str, &query_type));
+    array->set_query_type(query_type);
+  }
+
   return Status::Ok();
 }
 
